@@ -83,13 +83,24 @@ int	ResPortIndex;	/* Port ordering to backannotate into magic */
 extern ResSimNode *ResInitializeNode();
 extern CellUse	  *CmdGetSelectedCell();
 
+/* Linked list structure to use to store the substrate plane from each  */
+/* extracted CellDef so that they can be returned to the original after */
+/* extraction.								*/
+
+struct saveList {
+    Plane *sl_plane;
+    CellDef *sl_def;
+    struct saveList *sl_next;
+};
+
 /* Structure stores information required to be sent to ExtResisForDef() */
 typedef struct
 {
-    float	tolerance;
-    float	tdiTolerance;
-    float	frequency;
-    CellDef	*mainDef;
+    float	    tdiTolerance;
+    float	    frequency;
+    float	    rthresh;
+    struct saveList *savePlanes;
+    CellDef	    *mainDef;
 } ResisData;
 
 /*
@@ -107,13 +118,14 @@ ExtResisForDef(celldef, resisdata)
     CellDef *celldef;
     ResisData *resisdata;
 {
-    RDev	*oldRDev;
-    HashSearch	hs;
-    HashEntry	*entry;
-    devPtr	*tptr, *oldtptr;
+    RDev       *oldRDev;
+    HashSearch hs;
+    HashEntry  *entry;
+    devPtr     *tptr, *oldtptr;
     ResSimNode  *node;
-    int		result, idx;
-    char	*devname;
+    int                result, idx;
+    char       *devname;
+    Plane      *savePlane;
 
     ResRDevList = NULL;
     ResOriginalNodes = NULL;
@@ -122,22 +134,34 @@ ExtResisForDef(celldef, resisdata)
     if (HashLookOnly(&ResProcessedTable, celldef->cd_name)) return;
     HashFind(&ResProcessedTable, celldef->cd_name);
 
+    /* Prepare the substrate for resistance extraction */
+    savePlane = extResPrepSubstrate(celldef);
+    if (savePlane != NULL)
+    {
+	struct saveList *newsl;
+	newsl = (struct saveList *)mallocMagic(sizeof(struct saveList));
+	newsl->sl_plane = savePlane;
+	newsl->sl_def = celldef;
+	newsl->sl_next = resisdata->savePlanes;
+	resisdata->savePlanes = newsl;
+    }
+
     /* Get device information from the current extraction style */
     idx = 0;
     while (ExtGetDevInfo(idx++, &devname, NULL, NULL, NULL, NULL, NULL))
     {
-        if (idx == TT_MAXTYPES)
-        {
-            TxError("Error:  Ran out of space for device types!\n");
-            break;
-        }
-        efBuildAddStr(EFDevTypes, &EFDevNumTypes, TT_MAXTYPES, devname);
+	if (idx == TT_MAXTYPES)
+	{
+	    TxError("Error:  Ran out of space for device types!\n");
+	    break;
+	}
+	efBuildAddStr(EFDevTypes, &EFDevNumTypes, TT_MAXTYPES, devname);
     }
 
     HashInit(&ResNodeTable, INITFLATSIZE, HT_STRINGKEYS);
     /* read in .sim file */
     result = (ResReadSim(celldef->cd_name,
-	      	ResSimDevice, ResSimCapacitor, ResSimResistor,
+		ResSimDevice, ResSimCapacitor, ResSimResistor,
 		ResSimAttribute, ResSimMerge, ResSimSubckt) == 0);
 
     /* Clean up the EFDevTypes table */
@@ -188,7 +212,7 @@ ExtResisForDef(celldef, resisdata)
     HashKill(&ResNodeTable);
     while (ResRDevList != NULL)
     {
-    	oldRDev = ResRDevList;
+	oldRDev = ResRDevList;
 	ResRDevList = ResRDevList->nextDev;
 	if (oldRDev->layout != NULL)
 	{
@@ -199,7 +223,6 @@ ExtResisForDef(celldef, resisdata)
     }
 }
 
-
 /*
  *-------------------------------------------------------------------------
  *
@@ -217,16 +240,18 @@ ExtResisForDef(celldef, resisdata)
 
 void
 CmdExtResis(win, cmd)
-	MagWindow *win;
-	TxCommand *cmd;
+    MagWindow *win;
+    TxCommand *cmd;
 {
-    int		i, j, k, option, value, saveFlags;
-    static int	init = 1;
-    static float tolerance, tdiTolerance, fhFrequency;
+    int i, j, k, option, value, saveFlags;
+    static int init = 1;
+    static float rthresh, tdiTolerance, fhFrequency;
+
     CellDef	*mainDef;
     CellUse	*selectedUse;
     ResisData	resisdata;
     char	*endptr;	/* for use with strtod() */
+    struct saveList *sl;
 
     extern int resSubcircuitFunc();	/* Forward declaration */
 
@@ -239,8 +264,9 @@ CmdExtResis(win, cmd)
 
     static char *cmdExtresisCmd[] =
     {
-	"tolerance [value]    set ratio between resistor and device tol.",
 	"all 		       extract all the nets",
+	"threshold [value]    set minimum resistor extraction threshold",
+	"tolerance [value]    set ratio between resistor and device tol.",
 	"simplify [on/off]    turn on/off simplification of resistor nets",
 	"extout   [on/off]    turn on/off writing of .res.ext file",
 	"lumped   [on/off]    turn on/off writing of updated lumped resistances",
@@ -261,8 +287,9 @@ CmdExtResis(win, cmd)
     };
 
 typedef enum {
-	RES_BAD=-2, RES_AMBIG, RES_TOL,
-	RES_ALL, RES_SIMP, RES_EXTOUT, RES_LUMPED, RES_SILENT,
+	RES_BAD=-2, RES_AMBIG, RES_ALL,
+	RES_THRESH, RES_TOL,
+	RES_SIMP, RES_EXTOUT, RES_LUMPED, RES_SILENT,
 	RES_SKIP, RES_IGNORE, RES_INCLUDE, RES_BOX, RES_CELL,
 	RES_BLACKBOX, RES_FASTHENRY, RES_GEOMETRY, RES_HELP,
 #ifdef LAPLACE
@@ -278,7 +305,7 @@ typedef enum {
             TTMaskZero(&(ResCopyMask[i]));
 	    TTMaskSetMask(&ResCopyMask[i], &DBConnectTbl[i]);
      	}
-	tolerance = 1;
+	rthresh = 0;
 	tdiTolerance = 1;
 	fhFrequency = 10e6;	/* 10 MHz default */
 	HashInit(&ResIgnoreTable, INITFLATSIZE, HT_STRINGKEYS);
@@ -325,13 +352,12 @@ typedef enum {
 	    ResOptionsFlags |=  ResOpt_ExplicitRtol;
 	    if (cmd->tx_argc > 2)
 	    {
-		tolerance = MagAtof(cmd->tx_argv[2]);
-		if (tolerance <= 0)
+		tdiTolerance = MagAtof(cmd->tx_argv[2]);
+		if (tdiTolerance <= 0)
 		{
 		    TxError("Usage:  %s tolerance [value]\n", cmd->tx_argv[0]);
 			return;
 		}
-		tdiTolerance = tolerance;
 	    }
 	    else
 	    {
@@ -343,6 +369,26 @@ typedef enum {
 	    }
 	    return;
 
+	 case RES_THRESH:
+	    if (cmd->tx_argc > 2)
+	    {
+		rthresh = MagAtof(cmd->tx_argv[2]);
+		if (rthresh < 0)
+		{
+		    TxError("Usage:  %s threshold [value]\n", cmd->tx_argv[0]);
+			return;
+		}
+	    }
+	    else
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewDoubleObj((double)rthresh));
+#else
+		TxPrintf("Resistance threshold is %g.\n", rthresh);
+#endif
+	    }
+	    return;
+
 	 case RES_ALL:
 	    ResOptionsFlags |= ResOpt_ExtractAll;
 	    break;
@@ -623,10 +669,11 @@ typedef enum {
     ResOptionsFlags &= ~ResOpt_Power;
 #endif
 
-    resisdata.tolerance = tolerance;
+    resisdata.rthresh = rthresh;
     resisdata.tdiTolerance = tdiTolerance;
     resisdata.frequency = fhFrequency;
     resisdata.mainDef = mainDef;
+    resisdata.savePlanes = (struct saveList *)NULL;
 
     /* Do subcircuits (if any) first */
     HashInit(&ResProcessedTable, INITFLATSIZE, HT_STRINGKEYS);
@@ -636,6 +683,13 @@ typedef enum {
     ExtResisForDef(mainDef, &resisdata);
     HashKill(&ResProcessedTable);
 
+    /* Revert substrate planes */
+    for (sl = resisdata.savePlanes; sl; sl = sl->sl_next)
+    {
+	ExtRevertSubstrate(sl->sl_def, sl->sl_plane);
+	freeMagic(sl);
+    }
+
     /* turn back on undo stuff */
     UndoEnable();
 #ifdef LAPLACE
@@ -931,18 +985,29 @@ ResCheckSimNodes(celldef, resisdata)
 {
     ResSimNode	*node;
     devPtr	*ptr;
-    float	ftolerance, rctolerance, minRes, cumRes;
+    float	ftolerance, minRes, cumRes;
     int		failed1=0;
     int 	failed3=0;
     int		total =0;
     char	*outfile = celldef->cd_name;
-    float	tol = resisdata->tolerance;
     float	rctol = resisdata->tdiTolerance;
+    float	rthresh = resisdata->rthresh;
     int		nidx = 1, eidx = 1;	/* node & segment counters for geom. */
 
     if (ResOptionsFlags & ResOpt_DoExtFile)
     {
-	ResExtFile = PaOpen(outfile,"w",".res.ext",".",(char *) NULL, (char **) NULL);
+	if (ExtLocalPath != NULL)
+	    if (strcmp(ExtLocalPath, "."))
+	    {
+		char *namebuf;
+		namebuf = mallocMagic(strlen(ExtLocalPath) + strlen(celldef->cd_name)
+				+ 2);
+		sprintf(namebuf, "%s/%s", ExtLocalPath, celldef->cd_name);
+		outfile = namebuf;
+	    }
+	ResExtFile = PaOpen(outfile, "w", ".res.ext", ".", (char *)NULL, (char **)NULL);
+	if (outfile != celldef->cd_name) freeMagic(outfile);
+	outfile = celldef->cd_name;
     }
     else
     {
@@ -1122,16 +1187,10 @@ ResCheckSimNodes(celldef, resisdata)
 	    continue;
 	}
 	gparams.rg_bigdevres = (int)minRes * OHMSTOMILLIOHMS;
-	if ((rctol == 0.0) || (tol == 0.0))
-	{
-	    ftolerance = 0.0;
-	    rctolerance = 0.0;
-	}
+	if (minRes > resisdata->rthresh)
+	    ftolerance =  minRes;
 	else
-	{
-	    ftolerance =  minRes / tol;
-	    rctolerance = minRes / rctol;
-	}
+	    ftolerance = resisdata->rthresh; 
 
 	/*
 	 *   Is the device resistance greater than the lumped node
@@ -1151,17 +1210,16 @@ ResCheckSimNodes(celldef, resisdata)
 	    }
 	    else
 	    {
-		ResDoSimplify(ftolerance,rctol,&gparams);
+		ResDoSimplify(ftolerance, rctol, &gparams);
 		if (ResOptionsFlags & ResOpt_DoLumpFile)
 		{
 		    ResWriteLumpFile(node);
 		}
 		if (gparams.rg_maxres >= ftolerance  ||
-		        gparams.rg_maxres >= rctolerance ||
 			(ResOptionsFlags & ResOpt_ExtractAll))
 		{
 		    resNodeNum = 0;
-		    failed3 += ResWriteExtFile(celldef, node, tol, rctol,
+		    failed3 += ResWriteExtFile(celldef, node, rctol,
 				&nidx, &eidx);
 		}
 	    }
@@ -1322,10 +1380,29 @@ ResFixUpConnections(simDev, layoutDev, simNode, nodename)
 			layoutDev->rd_inside.r_xbot, layoutDev->rd_inside.r_ybot,
 			nodename);
     }
+
     if (simDev->source == simNode)
     {
+	/* Check for devices with only one terminal.  If it was cast as drain,	*/
+	/* then swap it with the source so that the code below handles it	*/
+	/* correctly.								*/
+
+	if (layoutDev->rd_fet_source == NULL && layoutDev->rd_fet_drain != NULL)
+	{
+	    layoutDev->rd_fet_source = layoutDev->rd_fet_drain;
+	    layoutDev->rd_fet_drain = (struct resnode *)NULL;
+	}
+
      	if (simDev->drain == simNode)
 	{
+	    if ((layoutDev->rd_fet_source != NULL) &&
+			(layoutDev->rd_fet_drain == NULL))
+	    {
+		/* Handle source/drain-tied devices */
+		if (simDev->drain == simDev->source)
+		    layoutDev->rd_fet_drain = layoutDev->rd_fet_source;
+	    }
+
 	    if (((source = layoutDev->rd_fet_source) != NULL) &&
 	       	   ((drain = layoutDev->rd_fet_drain) != NULL))
 	    {
@@ -1432,7 +1509,7 @@ ResFixUpConnections(simDev, layoutDev, simNode, nodename)
 		    resNodeNum--;
 		    notdecremented = FALSE;
 		}
-		ResFixDevName(newname,DRAIN,simDev,source);
+		ResFixDevName(newname, DRAIN, simDev, source);
 		source->rn_name = simDev->drain->name;
 	    }
 	}
@@ -1717,10 +1794,10 @@ ResAlignNodes(nodelist, reslist)
  */
 
 int
-ResWriteExtFile(celldef, node, tol, rctol, nidx, eidx)
+ResWriteExtFile(celldef, node, rctol, nidx, eidx)
     CellDef	*celldef;
     ResSimNode	*node;
-    float	tol, rctol;
+    float	rctol;
     int		*nidx, *eidx;
 {
     float	RCdev;
@@ -1730,7 +1807,7 @@ ResWriteExtFile(celldef, node, tol, rctol, nidx, eidx)
 
     RCdev = gparams.rg_bigdevres * gparams.rg_nodecap;
 
-    if (tol == 0.0 || (node->status & FORCE) ||
+    if ((node->status & FORCE) ||
 		(ResOptionsFlags & ResOpt_ExtractAll) ||
 		(ResOptionsFlags & ResOpt_Simplify) == 0 ||
 		(rctol + 1) * RCdev < rctol * gparams.rg_Tdi)
